<?php

class RadioactivityIncidentStorage {

  protected $id;

  protected $entityType;
  protected $fieldName;
  protected $bundle;
  protected $settings;

  /* db */
  protected $energyColumnName;
  protected $timestampColumnName;
  protected $tableName;

  /* d */
  protected $fieldInfo;

  /* This is the storage for default emitter behaviour */
  static $emitters;

  /* Time now */
  static $timeNow;

  /**
   * Constructor
   * @param unknown_type $field_id
   */
  public function __construct($field_info) {

    $this->fieldInfo = $field_info;

    $this->entityType = $field_info['entity_type'];
    $this->fieldId    = $field_info['field_id'];
    $this->bundle     = $field_info['bundle'];
    $this->settings   = $field_info['settings'];

    $this->fieldName           = $field_info['field_name'];
    $this->tableName           = "field_data_" . $this->fieldName;
    $this->energyColumnName    = $this->fieldName . '_' . RADIOACTIVITY_FIELD_ENERGY;
    $this->timestampColumnName = $this->fieldName . '_' . RADIOACTIVITY_FIELD_TIMESTAMP;

    if (self::$timeNow == NULL) {
      self::$timeNow = time();
    }

  }

  /**
   * Add incident to the storage system
   * @entity_energy Energies by entity id
   */
  public function addIncident(array $entity_energy) {
  }

  /**
   * Process incidents in the storage
   * When this is run depends solely on cron timing & decay pattern granularity
   */
  public function processIncidents() {
  }


  /**
   * Default emitting for the default processEmitter function
   */
  public function addEmitter($entity_id, $energy) {

    if (self::$emitters == NULL)
      self::$emitters = array();

    if ($entity_id !== NULL) {

      if (!isset(self::$emitters[$this->entityType][$this->bundle][$this->fieldName][$entity_id]))
        self::$emitters[$this->entityType][$this->bundle][$this->fieldName][$entity_id] = 0;

      self::$emitters[$this->entityType][$this->bundle][$this->fieldName][$entity_id] += $energy;

    } else {

      return self::$emitters;

    }
  }


  /**
   * Default emitter processing via ajax callback path
   */
  public function processEmitters() {
    if (count(self::$emitters) > 0) {
      $data = drupal_json_encode(self::$emitters);
      $checksum = md5($data . variable_get('radioactivity_ajax_salt', RADIOACTIVITY_AJAX_SALT));
      $path = base_path() . variable_get('radioactivity_ajax_callback_path', RADIOACTIVITY_AJAX_PATH);
      drupal_add_js(drupal_get_path('module', 'radioactivity') .'/js/radioactivity.js');
      drupal_add_js(array('radioactivity' => array('callback' => $path)), 'setting');
      drupal_add_js(array('radioactivity' => array('data' => $data)), 'setting');
      drupal_add_js(array('radioactivity' => array('checksum' => $checksum)), 'setting');
    }
  }

  /**
   * Get field info attached to this instance
   */
  public function getFieldInfo() {
    return $this->fieldInfo;
  }

  /**
   * Get decay profile attached to this instance
   */
  public function getDecayProfile() {
    return radioactivity_decay_profile_load($this->settings['profile']);
  }


  /**
   * Update field table for the given entity_id and shift energy with the given delta
   */
  function updateEnergy($entity_id, $energy_delta) {    
    db_merge($this->tableName)
      ->key(array(
        'entity_type' => $this->entityType,
        'bundle' => $this->bundle,
        'entity_id' => $entity_id,
        'language' => LANGUAGE_NONE,
        'delta' => 0,
      ))
      ->fields(array(
        'revision_id' => $entity_id,
        $this->timestampColumnName => self::$timeNow,
        $this->energyColumnName => $energy_delta,
      ))
      ->expression($this->energyColumnName, $this->energyColumnName . ' + :inc', array(':inc' => $energy_delta))
      ->execute();
  }


}
